#!/usr/bin/env Rscript

today <- format(Sys.time(), "%a_%b_%d")
cmd_args <- commandArgs(trailingOnly = T)

if (length(cmd_args) != 1)
{
  message = "Usage: EthSEQ.R <config_file>"
  cat(paste("[",Sys.time() ,"] ",message,"\n",sep=""))
  q()
}

verbose = FALSE
run.genotype = TRUE
reduce.composite.model = TRUE
composite.model.call.rate = 1
ped.target = ""

## Load configuration file
configFile <- cmd_args[1]

source(configFile)
source(paste(source.dir,"Functions.R",sep=""))
verbose = !verbose

## Plot intro message
EthSEQ_version<-"1.0"
message(paste("Running EthSEQ v",EthSEQ_version,sep=""))
message(paste("Configuration file: ",configFile,sep=""))
message(paste("Working directory: ",out.dir,sep=""))

## Select Model
if(model!="")
{
  res = get.Model(model,reference,ped.lab,vcf.lab,map.lab)
  model.ped = res[[1]]
  model.map = res[[2]]
  vcf.file = res[[3]]
  sif.file = res[[4]]
} else
{
 if(model.ped==""|model.map==""|vcf.file==""|sif.file=="")
 {
   error.message("Reference model files are missing.\n")
 }
}

bam.files = unique(as.character(read.delim(bam.list,header=F)[,1]))
sample.names = gsub(".bam","",basename(bam.files))

## Check length of names for EIGENSTRAT limit
name.lengths = as.numeric(sapply(sample.names,function(x) length(strsplit(x,"")[[1]])))
if(any(name.lengths>39))
{
  message = paste(sample.names[which(name.lengths>39)],collapse="\n")
  error.message(paste("There samples with ID length >39 characters:\n",message,sep=""))
}

## Execute ASEQ to generate genotypes
table = data.frame(sample=gsub(".bam","",basename(bam.files)),bam=bam.files)
genotype.dir = paste(out.dir,"ASEQGenotypes/",sep="")
if(!file.exists(out.dir))
  system(paste("mkdir ",out.dir),ignore.stdout=verbose,ignore.stderr=verbose)

if(run.genotype)
{
  message(paste("Run ASEQ to generate Dataset genotypes on ",length(bam.files)," samples",sep=""))
  aseq.run(bam.files,genotype.dir,mbq,mrq,mdc,vcf.file,cores)
}

## Create EIGENSTRAT folder
eigenstrat.dir = paste(out.dir,"EIGENSTRAT/",sep="")
if(!file.exists(eigenstrat.dir))
  system(paste("mkdir",eigenstrat.dir),ignore.stdout=verbose,ignore.stderr=verbose)

## Create Dataset PED
message("Create aggregated PED and MAP files")
if (ped.target=="")
{
  ped = create.PED(genotype.dir,reference)
} else
{
  ped = read.table(ped.target,sep="\t",as.is=T,header=F)
}
write.table(ped[,2:ncol(ped)],paste(out.dir,"Dataset.ped",sep=""),sep="\t",col.names=F,row.names=F,quote=F)
## Remove FFID in Reference model 
command = paste("awk -F\"\\t\" '{out=$2; for(i=3;i<=NF;i++){out=out\"\\t\"$i}; print out}' ",model.ped," > ",out.dir,"tmp.ped",sep="")
system(command,ignore.stderr=verbose)
## Merge Dataset and Reference model
command = paste("cat ",out.dir,"Dataset.ped ",out.dir,"tmp.ped > ",out.dir,"tmp2.ped",sep="")
system(command,ignore.stderr=verbose)
## Add FFID to aggregated model and mv to eigenstrat folder
command = paste("awk -F\"\\t\" '{out=NR\"\\t\"$1; for(i=2;i<=NF;i++){out=out\"\\t\"$i}; print out}' ",out.dir,"tmp2.ped > ",eigenstrat.dir,"DatasetReference.ped",sep="")
system(command,ignore.stderr=verbose)
## Remove temporary files
system(paste("rm ",out.dir,"Dataset.ped",sep=""),ignore.stderr=verbose)
system(paste("rm ",out.dir,"tmp.ped",sep=""),ignore.stderr=verbose)
system(paste("rm ",out.dir,"tmp2.ped",sep=""),ignore.stderr=verbose)
## Copy Reference MAP file to eigestrat folder
system(paste("cp ",model.map," ",eigenstrat.dir,"DatasetReference.map",sep=""),ignore.stderr=verbose)

call.rate = ped[,c(2,1)]
mapref = read.table(paste(eigenstrat.dir,"DatasetReference.map",sep=""),sep="\t",as.is=T,header=F)
n.snps = nrow(mapref)
if(reduce.composite.model)
{
  message("Composite model reduction")
  pedref = read.table(paste(eigenstrat.dir,"DatasetReference.ped",sep=""),sep="\t",as.is=T,header=F)
  idx = which(apply(pedref[,7:ncol(pedref)],2,function(x) (length(which(x!="0 0"))/length(x))>=composite.model.call.rate))
  message(paste("Remaining SNPs: ",length(idx),sep=""))
  pedref = pedref[,c(1:6,idx+6)]
  mapref = mapref[idx,]
  write.table(pedref,paste(eigenstrat.dir,"DatasetReference.ped",sep=""),sep="\t",row.names=F,quote=F,col.names=F)
  write.table(mapref,paste(eigenstrat.dir,"DatasetReference.map",sep=""),sep="\t",row.names=F,quote=F,col.names=F)
  call.rate[,2] = apply(pedref[1:length(bam.files),7:ncol(pedref)],1,function(x) length(which(x!="0 0"))/length(x))
  n.snps = as.character(length(idx))
}

## Run EIGENSTRAST
message("Run EIGENSTRAT")
setwd(eigenstrat.dir)
config = paste(
  "genotypename: DatasetReference.ped",
  "snpname: DatasetReference.map",
  "indivname: DatasetReference.ped",
  "outputformat: EIGENSTRAT",
  "genotypeoutname: DatasetReference.eigenstratgeno",
  "snpoutname: DatasetReference.snp",
  "indivoutname: DatasetReference.ind",
  "familynames: NO",
sep="\n")
write(config,paste(eigenstrat.dir,"par.PED.EIGENSTRAT",sep=""))
system(paste(eigenstrat.path,"convertf -p par.PED.EIGENSTRAT",sep=""),ignore.stdout=verbose,ignore.stderr=verbose)

config = paste(
  "genotypename: DatasetReference.eigenstratgeno",
  "snpname: DatasetReference.snp",
  "indivname: DatasetReference.ind",
  "evecoutname: DatasetReference.pca.evec",
  "evaloutname: DatasetReference.eval",
  "altnormstyle: NO",
  "numoutevec: 10",
  "numoutlieriter: 0",
  "numoutlierevec: 2",
  "outliersigmathresh: 6.0",
  sep="\n")
write(config,paste(eigenstrat.dir,"DatasetReference.pca.par",sep=""))
system(paste(eigenstrat.path,"smartpca -p DatasetReference.pca.par",sep=""),ignore.stdout=verbose,ignore.stderr=verbose)

## Generate reports
message("Generate Analysis Reports")
plot.Report(eigenstrat.dir,length(bam.files),call.rate,n.snps,sif.file)

message("Computation finished")

