## Run ASEQ to get genotype
aseq.run <- function(bam.files,out.dir,mbq,mrq,mdc,vcf.file,cores)
{
  for (b in bam.files)
  {
    command = paste(ASEQ.path," vcf=",vcf.file," bam=",b," mode=GENOTYPE threads=",cores," htperc=0.2 mbq=",mbq,
                    " mrq=",mrq," mdc=",mdc," out=",out.dir,sep="")
    system(command,ignore.stderr=verbose)
    if(verbose)
      cat("*")
  }
  if(verbose)
    cat("\n")
}

## Create PED of ASEQ genotype results
create.PED <- function(genotype.dir,reference)
{
  files = list.files(genotype.dir,"genotype.vcf")
  for(f in files)
  {
    if(verbose)
      cat("*")
    geno = read.table(paste(genotype.dir,f,sep=""),sep="\t",header=F,as.is=T)
    geno = geno[,10]
    call.rate = 100-round(length(which(geno=="./."))/length(geno)*100,digits=1)
    geno[which(geno=="./.")] = "0 0"
    geno[which(geno=="0/1")] = "1 2"
    geno[which(geno=="0/0")] = "1 1"
    geno[which(geno=="1/1")] = "2 2"
    if(f==files[1])
    {
      ped = c(call.rate,gsub(".genotype.vcf","",f),0,0,0,1,geno)
    } else
    {
      ped = rbind(ped,c(call.rate,gsub(".genotype.vcf","",f),0,0,0,1,geno))
    }
  }
  if(verbose)
    cat("*")
  return(ped)
}

plot.Report <- function(eigenstrat.dir,n.samples,call.rate,n.snps,sif.file)
{
  library(rgeos)
  setwd(eigenstrat.dir)
  
  ### Load PCA results
  pca_Allgenome<-read.table("DatasetReference.pca.evec", stringsAsFactors=F)
  
  ### Load SIF Datset
  tab = cbind(pca_Allgenome[,1],NA)
  names = c()
  for(i in 1:nrow(tab))
  {
    if(length(grep(tab[i,1],as.character(table$sample)))>0)
    {
      tab[i,2] = as.character(table$bam[grep(tab[i,1],as.character(table$sample))])
      names = cbind(names,as.character(table$sample[grep(tab[i,1],as.character(table$sample))])==tab[i,1])
    }
  }
  tab = tab[which(!is.na(tab[,2])),]
  if(!all(names))
    error.message("ERROR")
  
  ### Load Reference description
  idx.ref = (n.samples+1):nrow(pca_Allgenome)
  
  sif = read.table(paste(sif.file,sep=""),sep="\t",as.is=T,header=T)
  sif = sif[which(sif$Sample%in%pca_Allgenome[idx.ref,1]),]
  
  ### Print PCA schema
  populations = unique(sif$Race)
  populations = populations[!is.na(populations)]
  colors = rainbow(length(populations),alpha=0.8)
  polygons = list()
  
  cols = rep(rgb(0,0,0,alpha=0.8),nrow(pca_Allgenome))
  for(i in 1:length(populations))
  {
    ids = which(pca_Allgenome[,1]%in%sif$Sample[which(sif$Race==populations[i])])
    cols[ids] = colors[i]
    tmp = pca_Allgenome[ids,]
    hpts = chull(tmp[,2],tmp[,3])
    hpts = c(hpts, hpts[1])
    polygons[[length(polygons)+1]] = tmp[hpts,2:3]
  }
  
  tab = as.data.frame(tab)
  tab$V3 = NA
  tab$V4 = NA
  tab$V5 = ""
  tab$V6 = ""
  tab$V7 = "black"
  tab$V8 = NA
  tab$V9 = -1
  tab$V10 = ""
  
  for(i in 1:nrow(tab))
  {
    id = which(pca_Allgenome[,1]==tab[i,1]) 
    tab[i,3] = pca_Allgenome[id,2]
    tab[i,4] = pca_Allgenome[id,3]
    tab[i,8] = pca_Allgenome[id,4]
    
    for(j in 1:length(polygons))
    {
      polypnts = polygons[[j]] 
      pnts = data.frame(as.numeric(tab[i,3]),as.numeric(tab[i,4]))
      colnames(polypnts) = c("x","y")
      colnames(pnts) = c("x","y")
      
      polypnts = readWKT(paste("POLYGON((",paste(paste(polypnts$x,polypnts$y,sep=" "),collapse=","),"))",sep=""))
      pnts = readWKT(paste("POINT(",paste(paste(pnts$x,pnts$y,sep=" "),collapse=","),")",sep=""))
      res = gContains(polypnts,pnts)  
      if(res)
      {
        tab[i,5] = paste(c(strsplit(tab[i,5],"\\|")[[1]],populations[j]),collapse="|")
        tab[i,6] = "INSIDE"
      } else {
        if(tab[i,9]<0)
        {
          tab[i,9] = gDistance(polypnts,pnts)
          tab[i,10] = populations[j]
        } else {
          d = gDistance(polypnts,pnts)
          if(d < tab[i,9])
          {
            tab[i,9] = d
            tab[i,10] = populations[j]
          } else if(d == tab[i,9]) {
            tab[i,9] = d
            tab[i,10] = paste(c(strsplit(tab[i,10],"\\|")[[1]],populations[j]),collapse="|")
          }
        }
      }
    }
    
    if(tab[i,5]=="")
    {
      tab[i,5] = tab[i,10]
      tab[i,6] = "CLOSEST"
    }
    
    tab[i,7] = colors[which(populations==strsplit(as.character(tab$V5[i]),"\\|")[[1]][1])]
    
  }
  
  pdf("Report.pdf",11, 8)
  
  legend.space = (max(pca_Allgenome[,2])-min(pca_Allgenome[,2]))/7
  plot(NA,xlim=c(min(pca_Allgenome[,2])-legend.space,max(pca_Allgenome[,2])),ylim=c(min(pca_Allgenome[,3]),max(pca_Allgenome[,3])),
       main=paste("Dataset samples with Reference model populations (#SNPs=",n.snps,")",sep=""),xlab="PC1",ylab="PC2")
  points(as.numeric(tab[,3]),as.numeric(tab[,4]),pch=16,cex=0.6)
  for(i in 1:length(polygons))
  {
    lines(polygons[[i]][,1],polygons[[i]][,2],lwd=3,col=colors[i])
  }
  legend("bottomleft",legend=populations,pch=16, col=colors,bty = "n",cex=0.7)
  
  plot(NA,xlim=c(min(pca_Allgenome[,2])-legend.space,max(pca_Allgenome[,2])),ylim=c(min(pca_Allgenome[,3]),max(pca_Allgenome[,3])),
       main=paste("Dataset samples (#SNPs=",n.snps,")",sep=""),xlab="PC1",ylab="PC2")
  points(as.numeric(tab[,3]),as.numeric(tab[,4]),col=tab$V7,pch=16,cex=0.6)
  legend("bottomleft",legend=populations,pch=16, col=colors,bty = "n",cex=0.7)
  
  plot(NA,xlim=c(min(pca_Allgenome[,2])-legend.space,max(pca_Allgenome[,2])),ylim=c(min(pca_Allgenome[,3]),max(pca_Allgenome[,3])),
       main=paste("Reference model samples (#SNPs=",n.snps,")",sep=""),xlab="PC1",ylab="PC2")
  points(pca_Allgenome[idx.ref,2],pca_Allgenome[idx.ref,3],col=cols[idx.ref],pch=16,cex=0.6)
  legend("bottomleft",legend=populations,pch=16, col=colors,bty = "n",cex=0.7)
  
  dev.off()
  
  out.tab = tab[,c(1,2,5,6)]
  isort = match(out.tab[,1],call.rate[,1])
  call.rate = call.rate[isort,]
  all(call.rate[,1]==out.tab[,1])
  out.tab = cbind(out.tab,as.numeric(call.rate[,2]))
  
  colnames(out.tab) = c("sample","bam","ethnicity","type","call.rate")
  write.table(out.tab,"Report.csv",sep="\t",quote=F,row.names=F)
  
  pdf("Report_Detailed.pdf",11, 8)
  
  for(j in 1:nrow(tab))
  {
    legend.space = (max(pca_Allgenome[,2])-min(pca_Allgenome[,2]))/7
    plot(NA,xlim=c(min(pca_Allgenome[,2])-legend.space,max(pca_Allgenome[,2])),ylim=c(min(pca_Allgenome[,3]),max(pca_Allgenome[,3])),
         main=paste("Sample ",tab[j,1]," (#SNPs=",n.snps,")",sep=""),xlab="PC1",ylab="PC2")
    points(as.numeric(tab[j,3]),as.numeric(tab[j,4]),pch=16,cex=0.8)
    for(i in 1:length(polygons))
    {
      lines(polygons[[i]][,1],polygons[[i]][,2],lwd=3,col=colors[i])
    }
    legend("bottomleft",legend=populations,pch=16, col=colors,bty = "n",cex=0.7)
    text(min(pca_Allgenome[,2])-legend.space,max(pca_Allgenome[,3]),paste("Annotation: ",tab[j,5]," (call-rate ",out.tab$call.rate[j],"%)",sep=""),cex=0.8,adj=0)
  }
  
  dev.off()
  
}

## Set selected model
get.Model <- function(model,reference,ped.lab,vcf.lab,map.lab)
{
  models.available = c("SS2","SS4","HALO","NimblegenV3")

  if(model=="HALO") 
  {
    model.ped = paste(source.dir,"Models/HALO.Model.ped",sep="")  
    model.map = paste(source.dir,"Models/HALO.Model.map",sep="") 
    sif.file = paste(source.dir,"Models/HALO.Model.txt",sep="")
    vcf.file = paste(source.dir,"VCF/HALO.Model.vcf",sep="") 
  } 
  if(model=="SS4") 
  {
    model.ped = paste(source.dir,"Models/SS4.Model.ped",sep="")  
    model.map = paste(source.dir,"Models/SS4.Model.map",sep="")
    sif.file = paste(source.dir,"Models/SS4.Model.txt",sep="")
    vcf.file = paste(source.dir,"VCF/SS4.Model.vcf",sep="") 
  } 
  if(model=="SS2") 
  {
    model.ped = paste(source.dir,"Models/SS2.Model.ped",sep="")  
    model.map = paste(source.dir,"Models/SS2.Model.map",sep="")
    sif.file = paste(source.dir,"Models/SS2.Model.txt",sep="")
    vcf.file = paste(source.dir,"VCF/SS2.Model.vcf",sep="") 
  } 
  if(model=="NimblegenV3") 
  {
    model.ped = paste(source.dir,"Models/NimblegenV3.Model.ped",sep="")  
    model.map = paste(source.dir,"Models/NimblegenV3.Model.map",sep="")
    sif.file = paste(source.dir,"Models/NimblegenV3.Model.txt",sep="")
    vcf.file = paste(source.dir,"VCF/NimblegenV3.Model.vcf",sep="") 
  }
  if(!model%in%models.available)
  {
    error.message(paste("ERROR: Model ",model," not available",sep=""))
  }
  return(list(model.ped,model.map,vcf.file,sif.file))
}

## Message
message <- function(message)
{
  cat(paste("[",Sys.time() ,"] ",message,"\n",sep=""))
}

## Error message
error.message <- function(message)
{
  cat(paste("[",Sys.time() ,"] ",message,".\n",sep=""))
  q()
}

